from typing import List

from blue_options.terminal import show_usage, xtra


def help_vision(
    tokens: List[str],
    mono: bool,
) -> str:
    options = "".join(
        [
            "detail=<detail>",
            xtra(",dryrun,~upload", mono=mono),
        ]
    )

    image_options = "Davie,~Bute,.jpg"

    args = [
        "[--max_count <-1>]",
        "[--verbose 1]",
    ]

    return show_usage(
        [
            "@openai",
            "vision",
            f"[{options}]",
            f"[{image_options}]",
            '"<prompt>"',
            "[.|<object-name>]",
        ]
        + args,
        "complete <prompt> given the image(s) in <object-name>.",
        {
            "detail: auto | low | high": [],
        },
        mono=mono,
    )



from typing import List

from blue_options.terminal import show_usage, xtra
from blueflow.help.workflow import runner_details

from openai_commands.env import LITERATURE_REVIEW_OBJECT

review_args = [
    "[--count <-1>]",
    "[--filename <filename>]",
    "[--overwrite 1]",
    "[--verbose 1]",
]


def help_(
    tokens: List[str],
    mono: bool,
) -> str:

    options = "".join(
        [
            xtra("dryrun,~download,", mono=mono),
            "publish,question=<question>",
            xtra(",suffix=<suffix>,~upload", mono=mono),
        ]
    )

    return show_usage(
        [
            "@litrev",
            f"[{options}]",
            f"[{LITERATURE_REVIEW_OBJECT} | <object-name>]",
        ]
        + review_args,
        "ask a multiple-choice question about the list of studies in <object-name>.",
        {
            "input: <object-name>/<filename>.csv, column: Abstract.": [],
            "question: <object-name>/<question>.yaml.": [],
            "output: <object-name>-<suffix>-<question>/<filename>.csv, column: <question>.": [],
        },
        mono=mono,
    )


def help_combine(
    tokens: List[str],
    mono: bool,
) -> str:
    options = "".join(
        [
            xtra("~download,dryrun,", mono=mono),
            "publish",
            xtra(",~upload", mono=mono),
        ]
    )

    return show_usage(
        [
            "@litrev",
            "combine",
            f"[{options}]",
            "[...|<object-name-1>]",
            "[..|<object-name-2>]",
            "[.|<object-name>]",
        ],
        "<object-name-1> + <object-name-2> -> <object-name>.",
        mono=mono,
    )


def help_multiple(
    tokens: List[str],
    mono: bool,
) -> str:
    options = "".join(
        [
            xtra("dryrun,", mono=mono),
            "publish,questions=<question1+question2+...>",
            xtra(",suffix=<suffix>", mono=mono),
        ]
    )
    workflow_options = "".join(
        [
            xtra("dryrun,", mono=mono),
            "to=<runner>",
        ]
    )
    review_options = "".join(
        [
            xtra("dryrun,", mono=mono),
            "publish",
        ]
    )

    return show_usage(
        [
            "@litrev",
            "multiple",
            f"[{options}]",
            f"[{workflow_options}]",
            f"[{review_options}]",
            f"[{LITERATURE_REVIEW_OBJECT} | <object-name>]",
        ]
        + review_args,
        "ask multiple multiple-choice questions about the list of studies in <object-name>.",
        {
            "input: <object-name>/<filename>.csv, column: Abstract.": [],
            "questions: <object-name>/<question1>.yaml, <question2>.yaml, ... .": [],
            "output: <object-name>-<suffix>-<question1>-<question-2>-<...>/<filename>.csv, columns: <question1>, <question2>, ... .": [],
            **runner_details,
        },
        mono=mono,
    )


help_functions = {
    "": help_,
    "combine": help_combine,
    "multiple": help_multiple,
}



from blue_objects.env import ABCLI_PUBLIC_PREFIX
from blue_options import string


runner_prefix = "https://github.com/kamangir/notebooks-and-scripts/tree/main/blueflow/workflow/runners"

list_of_runners = ["aws_batch", "local"]

items = [
    f"[{runner_type}]({runner_prefix}/{runner_type}.py)"
    for runner_type in list_of_runners
] + [
    f"[![image]({url})]({url})"
    for url in [
        "{}/AMR-v7-test-litrev-multiple-{}-study-type-screening-result/workflow.gif?raw=true&random={}".format(
            ABCLI_PUBLIC_PREFIX,
            runner_type,
            string.random(),
        )
        for runner_type in list_of_runners
    ]
]



from typing import List

from blue_options.terminal import show_usage, xtra


def help_generate_image(
    tokens: List[str],
    mono: bool,
) -> str:
    options = "".join(
        [
            xtra("dryrun,", mono=mono),
            "filename=<filename.png>",
            xtra(",~upload", mono=mono),
        ]
    )

    args = [
        "[--verbose 1]",
    ]

    return show_usage(
        [
            "@openai",
            "generate_image",
            f"[{options}]",
            '"<prompt>"',
            "[-|<object-name>]",
        ]
        + args,
        "<prompt> -generate-image-> <object-name>/<filename.png>.",
        mono=mono,
    )



from typing import Tuple, Any, Dict
from openai import OpenAI
from blueness import module
from blue_options.host import is_jupyter
from openai_commands import env
from openai_commands import NAME
from openai_commands.logger import logger

NAME = module.name(__file__, NAME)


# https://github.com/openai/openai-python
# https://github.com/openai/openai-python/discussions/742
def complete_prompt(
    prompt: str,
    max_tokens: int = 2000,
    verbose=None,
) -> Tuple[bool, str, Dict[str, Any]]:
    if not env.OPENAI_API_KEY:
        logger.error("OPENAI_API_KEY is not set.")
        return False, "", {}

    client = OpenAI(api_key=env.OPENAI_API_KEY)

    try:
        response = client.chat.completions.create(
            messages=[
                {
                    "role": "user",
                    "content": prompt,
                }
            ],
            model=env.OPENAI_GPT_DEFAULT_MODEL,
            max_tokens=max_tokens,
        )
    except Exception as e:
        logger.info(
            "{}.complete_prompt failed: {}, prompt length: {}".format(
                NAME,
                str(e),
                len(prompt),
            )
        )
        return False, str(e), {"exception": str(e)}

    if is_jupyter() if verbose is None else verbose:
        logger.info("response: {}".format(response))

    if not response.choices:
        logger.info(f"{NAME}.complete(): no choice.")
        return False, "", {"status": "no choice"}

    if len(response.choices) > 1 and verbose:
        logger.info(
            "{} choices, picked the first, and ignored the rest.".format(
                len(response.choices)
            )
        )

    choice = response.choices[0]

    metadata = {
        "response": response,
        "status": f"choice: {choice.finish_reason}",
    }

    if verbose:
        logger.info(f"{NAME}.complete(): finish_reason: {choice.finish_reason}.")
    return choice.finish_reason == "stop", choice.message.content, metadata



from typing import List

from abcli.help.generic import help_functions as generic_help_functions
from blue_options.terminal import show_usage

from openai_commands import ALIAS
from openai_commands.help.image_generation import help_generate_image
from openai_commands.help.literature_review import (
    help_functions as help_literature_review,
)
from openai_commands.help.prompt_completion import help_complete
from openai_commands.help.vision import help_vision


def help_browse(
    tokens: List[str],
    mono: bool,
) -> str:
    options = "actions | dashboard"

    return show_usage(
        [
            "@openai",
            "browse",
            f"[{options}]",
        ],
        "browse @openai.",
        mono=mono,
    )


help_functions = generic_help_functions(plugin_name=ALIAS)

help_functions.update(
    {
        "browse": help_browse,
        "complete": help_complete,
        "generate_image": help_generate_image,
        "literature_review": help_literature_review,
        "vision": help_vision,
    }
)



import os

from blue_objects import file, README

from openai_commands.literature_review.README import items as literature_review_items
from openai_commands import NAME, VERSION, ICON, REPO_NAME


default_thumbnail = (
    "https://github.com/kamangir/assets/raw/main/blue-plugin/marquee.png?raw=true"
)

features = {
    "literature review": {
        "thumbnail": "https://github.com/kamangir/assets/blob/main/openai_commands/literature-review/marquee.png?raw=true",
        "url": "./openai_commands/literature_review",
    },
    "prompt completion": {
        "thumbnail": default_thumbnail,
        "url": "./openai_commands/prompt_completion",
    },
    "image generation": {
        "thumbnail": "https://github.com/kamangir/openai-commands/blob/main/assets/DALL-E.png?raw=true",
        "url": "./openai_commands/images",
    },
    "vision": {
        "thumbnail": "https://raw.githubusercontent.com/kamangir/assets/main/vanwatch/2023-11-25-openai-vision/ButeNorthDavie.jpg",
        "url": "./openai_commands/vision",
    },
    "template": {
        "description": "",
        "thumbnail": default_thumbnail,
        "url": "",
    },
}


items = [
    "[`{}`]({}) [![image]({})]({}) {}".format(
        feature,
        details["url"],
        details["thumbnail"],
        details["url"],
        details.get("description", ""),
    )
    for feature, details in features.items()
    if feature != "template"
]


def build():
    return all(
        README.build(
            items=readme.get("items", []),
            cols=readme.get("cols", 3),
            path=os.path.join(file.path(__file__), readme["path"]),
            NAME=NAME,
            VERSION=VERSION,
            REPO_NAME=REPO_NAME,
        )
        for readme in [
            {
                "items": items,
                "cols": 2,
                "path": "..",
            },
            {
                "items": literature_review_items,
                "cols": 2,
                "path": "literature_review",
            },
            {
                "path": "image_generation",
            },
        ]
    )



import argparse

from blueness import module
from blueness.argparse.generic import sys_exit

from openai_commands import NAME
from openai_commands.prompt_completion.api import complete_prompt
from openai_commands.logger import logger

NAME = module.name(__file__, NAME)


parser = argparse.ArgumentParser(NAME)
parser.add_argument(
    "task",
    type=str,
    help="complete",
)
parser.add_argument(
    "--filename",
    type=str,
)
parser.add_argument(
    "--prompt",
    type=str,
)
parser.add_argument(
    "--max_token",
    type=int,
    default=2000,
)
parser.add_argument(
    "--verbose",
    type=int,
    help="0|1",
    default=0,
)
args = parser.parse_args()

success = False
if args.task == "complete":
    success, text, _ = complete_prompt(
        args.prompt,
        args.max_token,
        args.verbose == 1,
    )
    print(text)
else:
    success = None

sys_exit(logger, NAME, args.task, success)



from typing import List

from abcli.host import signature as abcli_signature
from blueflow import fullname as blueflow_fullname

from openai_commands import fullname


def signature() -> List[str]:
    return [
        fullname(),
        blueflow_fullname(),
    ] + abcli_signature()



from typing import Dict
import re
import pandas as pd
from tqdm import tqdm

from blueness import module
from blue_objects import file, objects

from openai_commands import NAME
from openai_commands.prompt_completion.api import complete_prompt
from openai_commands.logger import logger


NAME = module.name(__file__, NAME)


def clean_prompt(prompt):
    return re.sub(r"\s+", " ", prompt.strip())


def generate_prompt(question: Dict[str, Dict]) -> str:
    choices = question.get("choices", {})
    description = question.get(
        "description",
        "read this abstract of a scentific paper",
    )

    logger.info(
        "{} @ {} choice(s): {}".format(
            description,
            len(choices),
            choices,
        )
    )

    prompt = """
    {} and return the correct choice from the following list followed with "because" and 
    then describe the reason for this choice in less than ten words. choices: {}
    """.format(
        description,
        " ".join(
            [
                f'"{choice}" {description}'
                for choice, description in choices.items()
                if choice != "otherwise"
            ]
            + ['"not relevant" if none of the above choices is correct."']
        ),
    )

    prompt = clean_prompt(prompt)

    logger.info(f"prompt: {prompt}")
    return prompt


def review_literature(
    input_object_name: str,
    output_object_name: str,
    filename: str,
    question: str,
    count: int,
    overwrite: bool = False,
    verbose: bool = False,
) -> bool:
    if filename.endswith(".csv"):
        filename = filename.split(".csv")[0]

    logger.info(
        "{}.review_literature: {}/{}.csv -{}-{}{}> {}/{}-{}.csv[{}]".format(
            NAME,
            input_object_name,
            filename,
            question,
            "" if count == -1 else f"{count}X-",
            "" if not overwrite else "overwrite-",
            output_object_name,
            filename,
            question,
            question,
        )
    )

    success, question_dict = file.load_yaml(
        objects.path_of(f"{question}.yaml", input_object_name)
    )
    if not success:
        return success

    output_filename = objects.path_of(
        f"{filename}.csv",
        output_object_name,
        create=True,
    )
    input_filename = objects.path_of(
        f"{filename}.csv",
        input_object_name,
    )
    if not overwrite and file.exists(output_filename):
        logger.info(
            "continuing from a previous run: {} ...".format(file.name(output_filename))
        )
        input_filename = output_filename

    success, df = file.load_dataframe(input_filename, log=True)
    if not success:
        return success

    prompt = generate_prompt(question_dict)

    if question not in df.columns:
        df[question] = pd.NA
        logger.info("added column: {}".format(question))

    counter = 0
    for idx in tqdm(df.index):
        assessment = df.loc[idx, question]
        if (
            not overwrite
            and not pd.isna(assessment)
            and assessment != ""
            and not assessment.startswith("failure")
        ):
            logger.info(f"✅ {assessment}")
            continue

        abstract = str(df.loc[idx, "Abstract"])

        success, assessment, _ = complete_prompt(
            clean_prompt(f"{prompt} {abstract}"),
            verbose=verbose,
        )
        if not success:
            assessment = f"failure: {assessment}"

        logger.info(
            " | ".join(
                [assessment]
                + (
                    [abstract]
                    if verbose
                    else [
                        "{} ... ".format(abstract[:100]),
                    ]
                )
            )
        )

        df.loc[idx, question] = assessment

        counter += 1
        if count != -1 and counter >= count:
            break
    logger.info(f"processed {counter:,} item(s)")

    return file.save_csv(
        output_filename,
        df,
        log=True,
    )



from typing import List

from blue_options.terminal import show_usage


def help_complete(
    tokens: List[str],
    mono: bool,
) -> str:
    args = [
        "[--max_tokens <2000>]",
        "[--verbose 1]",
    ]

    return show_usage(
        [
            "@openai",
            "complete",
            '"<prompt>"',
        ]
        + args,
        "complete <prompt>.",
        mono=mono,
    )



import json
from openai import OpenAI
from typing import Tuple, Any
from IPython.display import Image, display

from blueness import module
from blue_options import string
from blue_options.host import is_jupyter
from blue_objects import file, graphics, objects, path
from blue_objects.graphics.signature import sign_filename

from openai_commands import NAME, VERSION
from openai_commands.host import signature
from openai_commands import env
from openai_commands.logger import logger


NAME = module.name(__file__, NAME)


class OpenAIImageGenerator:
    def __init__(
        self,
        model="dall-e-3",
        verbose: bool = False,
    ):
        assert env.OPENAI_API_KEY

        self.client = OpenAI(api_key=env.OPENAI_API_KEY)
        self.verbose = verbose
        self.model = model
        logger.info(
            "{}[{}]".format(
                self.__class__.__name__,
                self.model,
            )
        )

    def generate(
        self,
        prompt: str,
        filename: str = "",
        sign: bool = True,
        line_width: int = 80,
        quality: str = "standard",
        size: str = "1024x1024",
    ) -> Tuple[bool, Any]:
        object_name = path.name(file.path(filename))

        logger.info(
            "{}.generate: {} -> {}/{}".format(
                self.__class__.__name__,
                prompt,
                object_name,
                file.name_and_extension(filename),
            )
        )

        response = self.client.images.generate(
            model=self.model,
            prompt=prompt,
            size=size,
            quality=quality,
            n=1,
        )

        logger.info(json.dumps(response.model_dump(), indent=4))

        success = file.download(response.data[0].url, filename) if filename else True

        if success and sign:
            success, image = file.load_image(filename)

        if success and sign:
            success = sign_filename(
                filename=filename,
                header=[
                    " | ".join(
                        objects.signature(
                            info=file.name_and_extension(filename),
                            object_name=object_name,
                        )
                        + [
                            string.pretty_shape_of_matrix(image),
                            f"quality: {quality}",
                        ]
                    ),
                ],
                footer=[
                    " | ".join(
                        [
                            f"prompt: {prompt}",
                            f"revised prompt: {str(response.data[0].revised_prompt)}",
                            f"model: {self.model}",
                        ]
                        + signature()
                    ),
                ],
                line_width=line_width,
            )

        if success and self.verbose and is_jupyter():
            display(Image(filename=filename))

        return (
            success,
            response,
        )
